#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <math.h>

#include "hid.h"

hid_context device;

typedef union 
{
	struct 
	{
		uint8_t bOperation;
		uint8_t bDevAddr;
		uint8_t bAddr;
		uint8_t bLen;
		uint8_t abData[];
	} pc2dev;
	
	struct 
	{
		uint8_t bRetStatus;
		uint8_t bRes1;
		uint8_t bRes2;
		uint8_t bLen;
		uint8_t abData[];
	} dev2pc;
	
	struct 
	{
		uint8_t bRetStatus;
		uint8_t bRes1;
		uint8_t bRes2;
		uint8_t bLen;
		float h;
		float t;
	} dev2pc_dht;
	
	uint8_t abBuffer[64];
} io_data_t;

#define OP_GET_DHT22         0x2A
#define OP_GET_DHT22_2       0x2B
#define OP_GET_DHT_RAW       0x2C

/*
	get_raw_data()
	    .
 */
int get_dht_data(float *h, float *t)
{
	int ret;
	unsigned char buffer[64];
	io_data_t *data_io = (io_data_t*)buffer;
	
	data_io->pc2dev.bOperation = OP_GET_DHT22;
	data_io->pc2dev.bDevAddr = 0;
	data_io->pc2dev.bAddr = 0;
	data_io->pc2dev.bLen = 0;
	
	ret = hid_write(&device, buffer, 64);
	if (ret == 0)
	{
		return -1;
	}
	
rep_read_answer_w:
	ret = hid_read(&device, buffer, 64);
	if (ret == 0)
	{
		return -1;
	}
	
	if (data_io->dev2pc.bRetStatus == 0xFF)
	{
		goto rep_read_answer_w;
	}
	
	*h = data_io->dev2pc_dht.h; 
	*t = data_io->dev2pc_dht.t;
	
	return data_io->dev2pc.bRetStatus;
}

/*
	get_raw_data()
	    .
 */
int get_dht2_data(float *h, float *t)
{
	int ret;
	unsigned char buffer[64];
	io_data_t *data_io = (io_data_t*)buffer;
	
	data_io->pc2dev.bOperation = OP_GET_DHT22_2;
	data_io->pc2dev.bDevAddr = 0;
	data_io->pc2dev.bAddr = 0;
	data_io->pc2dev.bLen = 0;
	
	ret = hid_write(&device, buffer, 64);
	if (ret == 0)
	{
		return -1;
	}
	
rep_read_answer_w:
	ret = hid_read(&device, buffer, 64);
	if (ret == 0)
	{
		return -1;
	}
	
	if (data_io->dev2pc.bRetStatus == 0xFF)
	{
		goto rep_read_answer_w;
	}
	
	*h = data_io->dev2pc_dht.h; 
	*t = data_io->dev2pc_dht.t;
	
	return data_io->dev2pc.bRetStatus;
}

uint8_t bits[5];

int dht_readSensor(uint8_t wakeupDelay)
{
	int ret;
	unsigned char buffer[64];
	io_data_t *data_io = (io_data_t*)buffer;
	
	data_io->pc2dev.bOperation = OP_GET_DHT_RAW;
	data_io->pc2dev.bDevAddr = 0;
	data_io->pc2dev.bAddr = 0;
	data_io->pc2dev.bLen = 0;
	
	ret = hid_write(&device, buffer, 64);
	if (ret == 0)
	{
		return -1;
	}
	
rep_read_answer_w:
	ret = hid_read(&device, buffer, 64);
	if (ret == 0)
	{
		return -1;
	}
	
	if (data_io->dev2pc.bRetStatus == 0xFF)
	{
		goto rep_read_answer_w;
	}
	
	memcpy(bits, data_io->dev2pc.abData, 5);
	
	return data_io->dev2pc.bRetStatus;
}

#define WORD(a,b) ((((int16_t)a)<<8)|(b))

int main(int argc, char **argv)
{
	float h, t;
	int ret;
	
	uint8_t sum;
	
	printf("Open device... ");
	ret = hid_find(&device, 0x1FC9, 0x0007, 0xFF00);
	if (ret == 0)
	{
		printf("ERROR: Device not found!\r\n");
		return 0;
	}
	
#if 1
	ret = get_dht_data(&h, &t);
	if (ret)
	{
		printf("ERROR: get_dht_data() return %d\r\n", ret);
		//goto close_and_exit;
	}
	
	printf("Sensor data:\r\n");
	printf("H: %.1f\r\n", h);
	printf("T: %.1f\r\n", t);
	
	//ret = get_dht2_data(&h, &t);
	//if (ret)
	//{
	//	printf("ERROR: get_dht2_data() return %d\r\n", ret);
	//	//goto close_and_exit;
	//}
	//
	//printf("Sensor data:\r\n");
	//printf("H: %f\r\n", h);
	//printf("T: %f\r\n", t);
#endif

	//ret = dht_readSensor(2);
	//if (ret)
	//{
	//	printf("ERROR: dht_readSensor() return %d\r\n", ret);
	//	//goto close_and_exit;
	//}
	//
	//printf("Sensor data: %02x %02x %02x %02x %02x\r\n", 
	//	bits[0], bits[1], bits[2], bits[3], bits[4]);
    //
	//// CONVERT AND STORE
	//h = WORD(bits[0], bits[1]) * 0.1;
	//t = WORD(bits[2]&0x7F, bits[3]) * 0.1;
	//if (bits[2] & 0x80)  // negative temperature
	//{
	//	t = -t;
	//}
    //
	//// TEST CHECKSUM
	//sum = bits[0] + bits[1] + bits[2] + bits[3];
	//if (bits[4] != sum)
	//{
	//	printf("Bad checksum!\r\n");
	//}
	//
	//printf("Sensor data:\r\n");
	//printf("H: %.1f\r\n", h);
	//printf("T: %.1f\r\n", t);

close_and_exit:
	hid_close(&device);

	return 0;
}
